@@ -19,14 +19,17 @@
 
 /**
  * SECTION:element-curlhttpsink
+ * @title: curlhttpsink
  * @short_description: sink that uploads data to a server using libcurl
  * @see_also:
  *
  * This is a network sink that uses libcurl as a client to upload data to
  * an HTTP server.
  *
- * <refsect2>
- * <title>Example launch line (upload a JPEG file to an HTTP server)</title>
+ * ## Example launch line
+ *
+ * Upload a JPEG file to an HTTP server.
+ *
  * |[
  * gst-launch-1.0 filesrc location=image.jpg ! jpegparse ! curlhttpsink  \
  *     file-name=image.jpg  \
@@ -35,7 +38,6 @@
  *     content-type=image/jpeg  \
  *     use-content-length=false
  * ]|
- * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -458,13 +460,19 @@ gst_curl_http_sink_set_mime_type (GstCurlBaseSink * bcsink, GstCaps * caps)
   GstStructure *structure;
   const gchar *mime_type;
 
-  if (sink->content_type != NULL) {
-    return;
-  }
-
   structure = gst_caps_get_structure (caps, 0);
   mime_type = gst_structure_get_name (structure);
-  sink->content_type = g_strdup (mime_type);
+
+  g_free (sink->content_type);
+  if (!g_strcmp0 (mime_type, "multipart/form-data") &&
+      gst_structure_has_field_typed (structure, "boundary", G_TYPE_STRING)) {
+    const gchar *boundary;
+
+    boundary = gst_structure_get_string (structure, "boundary");
+    sink->content_type = g_strconcat (mime_type, "; boundary=", boundary, NULL);
+  } else {
+    sink->content_type = g_strdup (mime_type);
+  }
 }
 
 static gboolean
