@@ -675,6 +675,12 @@ export_srtp_keys (GstDtlsConnection * self)
 
   profile = SSL_get_selected_srtp_profile (self->priv->ssl);
 
+  if (!profile) {
+    GST_WARNING_OBJECT (self,
+        "no srtp capabilities negotiated during handshake");
+    return;
+  }
+
   GST_INFO_OBJECT (self, "keys received, profile is %s", profile->name);
 
   switch (profile->id) {
@@ -688,7 +694,7 @@ export_srtp_keys (GstDtlsConnection * self)
       break;
     default:
       GST_WARNING_OBJECT (self, "invalid crypto suite set by handshake");
-      goto beach;
+      return;
   }
 
   client_key.key = exported_keys.client_key;
@@ -708,7 +714,6 @@ export_srtp_keys (GstDtlsConnection * self)
         auth);
   }
 
-beach:
   self->priv->keys_exported = TRUE;
 }
 
@@ -989,7 +994,7 @@ bio_method_ctrl (BIO * bio, int cmd, long arg1, void *arg2)
       GST_LOG_OBJECT (self, "BIO: EOF reset");
       return 1;
     case BIO_CTRL_EOF:{
-      gint eof = !(priv->bio_buffer_len - priv->bio_buffer_offset);
+      gint eof = priv->is_alive == FALSE;
       GST_LOG_OBJECT (self, "BIO: EOF query returned %d", eof);
       return eof;
     }
