@@ -205,10 +205,8 @@ gst_modplug_class_init (GstModPlugClass * klass)
 
   gstelement_class->change_state = gst_modplug_change_state;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&modplug_sink_template_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&modplug_src_template_factory));
+  gst_element_class_add_static_pad_template (gstelement_class, &modplug_sink_template_factory);
+  gst_element_class_add_static_pad_template (gstelement_class, &modplug_src_template_factory);
 
   gst_element_class_set_static_metadata (gstelement_class, "ModPlug",
       "Codec/Decoder/Audio", "Module decoder based on modplug engine",
@@ -301,11 +299,15 @@ gst_modplug_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_parse_position (query, &format, NULL);
       if (format == GST_FORMAT_TIME) {
         gint64 pos;
+        guint32 max;
 
-        pos = (modplug->song_length * modplug->mSoundFile->GetCurrentPos ());
-        pos /= modplug->mSoundFile->GetMaxPosition ();
-        gst_query_set_position (query, format, pos);
-        res = TRUE;
+	max = modplug->mSoundFile->GetMaxPosition();
+	if (max > 0) {
+          pos = (modplug->song_length * modplug->mSoundFile->GetCurrentPos ()) /
+            max;
+          gst_query_set_position (query, format, pos);
+          res = TRUE;
+        }
       }
     }
       break;
