@@ -5,6 +5,7 @@
  * Copyright (C) 2008 Michael Sheldon <mike@mikeasoft.com>
  * Copyright (C) 2011 Stefan Sauer <ensonic@users.sf.net>
  * Copyright (C) 2014 Robert Jobbagy <jobbagy.robert@gmail.com>
+ * Copyright (C) 2018 Nicola Murino <nicola.murino@gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -82,9 +83,6 @@ using namespace std;
 
 #include "gstfacedetect.h"
 #include <opencv2/imgproc.hpp>
-#if (CV_MAJOR_VERSION >= 4)
-#include <opencv2/imgproc/imgproc_c.h>
-#endif
 
 GST_DEBUG_CATEGORY_STATIC (gst_face_detect_debug);
 #define GST_CAT_DEFAULT gst_face_detect_debug
@@ -236,10 +234,10 @@ static void gst_face_detect_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
 static gboolean gst_face_detect_set_caps (GstOpencvVideoFilter * transform,
-    gint in_width, gint in_height, gint in_depth, gint in_channels,
-    gint out_width, gint out_height, gint out_depth, gint out_channels);
+    gint in_width, gint in_height, int in_cv_type,
+    gint out_width, gint out_height, int out_cv_type);
 static GstFlowReturn gst_face_detect_transform_ip (GstOpencvVideoFilter * base,
-    GstBuffer * buf, IplImage * img);
+    GstBuffer * buf, Mat img);
 
 static CascadeClassifier *gst_face_detect_load_profile (GstFaceDetect *
     filter, gchar * profile);
@@ -250,8 +248,7 @@ gst_face_detect_finalize (GObject * obj)
 {
   GstFaceDetect *filter = GST_FACE_DETECT (obj);
 
-  if (filter->cvGray)
-    cvReleaseImage (&filter->cvGray);
+  filter->cvGray.release ();
 
   g_free (filter->face_profile);
   g_free (filter->nose_profile);
@@ -517,18 +514,14 @@ gst_face_detect_get_property (GObject * object, guint prop_id,
 /* this function handles the link with other elements */
 static gboolean
 gst_face_detect_set_caps (GstOpencvVideoFilter * transform, gint in_width,
-    gint in_height, gint in_depth, gint in_channels,
-    gint out_width, gint out_height, gint out_depth, gint out_channels)
+    gint in_height, int in_cv_type,
+    gint out_width, gint out_height, int out_cv_type)
 {
   GstFaceDetect *filter;
 
   filter = GST_FACE_DETECT (transform);
 
-  if (filter->cvGray)
-    cvReleaseImage (&filter->cvGray);
-
-  filter->cvGray = cvCreateImage (cvSize (in_width, in_height), IPL_DEPTH_8U,
-      1);
+  filter->cvGray.create (Size (in_width, in_height), CV_8UC1);
 
   return TRUE;
 }
@@ -561,15 +554,15 @@ gst_face_detect_run_detector (GstFaceDetect * filter,
 {
   double img_stddev = 0;
   if (filter->min_stddev > 0) {
-    CvScalar mean, stddev;
-    cvAvgSdv (filter->cvGray, &mean, &stddev, NULL);
+    Scalar mean, stddev;
+    meanStdDev (filter->cvGray, mean, stddev);
     img_stddev = stddev.val[0];
   }
   if (img_stddev >= filter->min_stddev) {
-    Mat roi (cv::cvarrToMat (filter->cvGray), r);
+    Mat roi (filter->cvGray, r);
     detector->detectMultiScale (roi, faces, filter->scale_factor,
-        filter->min_neighbors, filter->flags, cvSize (min_size_width,
-            min_size_height), cvSize (0, 0));
+        filter->min_neighbors, filter->flags, Size (min_size_width,
+            min_size_height), Size (0, 0));
   } else {
     GST_LOG_OBJECT (filter,
         "Calculated stddev %f lesser than min_stddev %d, detection not performed",
@@ -582,7 +575,7 @@ gst_face_detect_run_detector (GstFaceDetect * filter,
  */
 static GstFlowReturn
 gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
-    IplImage * img)
+    Mat img)
 {
   GstFaceDetect *filter = GST_FACE_DETECT (base);
 
@@ -597,14 +590,13 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
     vector < Rect > eyes;
     gboolean post_msg = FALSE;
 
-    Mat mtxOrg (cv::cvarrToMat (img));
-
-    cvCvtColor (img, filter->cvGray, CV_RGB2GRAY);
+    cvtColor (img, filter->cvGray, COLOR_RGB2GRAY);
 
     gst_face_detect_run_detector (filter, filter->cvFaceDetect,
         filter->min_size_width, filter->min_size_height,
-        Rect (filter->cvGray->origin, filter->cvGray->origin,
-            filter->cvGray->width, filter->cvGray->height), faces);
+        Rect (0, 0,
+            filter->cvGray.size ().width, filter->cvGray.size ().height),
+        faces);
 
     switch (filter->updates) {
       case GST_FACEDETECT_UPDATES_EVERY_FRAME:
@@ -716,7 +708,7 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
       }
 
       if (filter->display) {
-        CvPoint center;
+        Point center;
         Size axes;
         gdouble w, h;
         gint cb = 255 - ((i & 3) << 7);
@@ -729,7 +721,7 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
         center.y = cvRound ((r.y + h));
         axes.width = w;
         axes.height = h * 1.25; /* tweak for face form */
-        ellipse (mtxOrg, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 3, 8, 0);
+        ellipse (img, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 3, 8, 0);
 
         if (have_nose) {
           Rect sr = nose[0];
@@ -740,8 +732,7 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
           center.y = cvRound ((rny + sr.y + h));
           axes.width = w;
           axes.height = h * 1.25;       /* tweak for nose form */
-          ellipse (mtxOrg, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 1, 8,
-              0);
+          ellipse (img, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 1, 8, 0);
         }
         if (have_mouth) {
           Rect sr = mouth[0];
@@ -752,8 +743,7 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
           center.y = cvRound ((rmy + sr.y + h));
           axes.width = w * 1.5; /* tweak for mouth form */
           axes.height = h;
-          ellipse (mtxOrg, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 1, 8,
-              0);
+          ellipse (img, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 1, 8, 0);
         }
         if (have_eyes) {
           Rect sr = eyes[0];
@@ -764,8 +754,7 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
           center.y = cvRound ((rey + sr.y + h));
           axes.width = w * 1.5; /* tweak for eyes form */
           axes.height = h;
-          ellipse (mtxOrg, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 1, 8,
-              0);
+          ellipse (img, center, axes, 0, 0, 360, Scalar (cr, cg, cb), 1, 8, 0);
         }
       }
       gst_buffer_add_video_region_of_interest_meta (buf, "face",
@@ -778,7 +767,6 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
       g_value_unset (&facelist);
       gst_element_post_message (GST_ELEMENT (filter), msg);
     }
-    mtxOrg.release ();
   }
 
   return GST_FLOW_OK;
