@@ -64,7 +64,7 @@
  * unless the user has set the relevant properties first. It also uses
  * a master key that MUST be set by property (key) at the beginning. The
  * master key must be of a maximum length of 46 characters (14 characters
- * for the salt plus the key). The encryption and authentication mecanisms
+ * for the salt plus the key). The encryption and authentication mechanisms
  * available are :
  *
  * Encryption (properties rtp-cipher and rtcp-cipher)
@@ -490,8 +490,10 @@ static void
 gst_srtp_enc_reset_no_lock (GstSrtpEnc * filter)
 {
   if (!filter->first_session) {
-    srtp_dealloc (filter->session);
-    filter->session = NULL;
+    if (filter->session) {
+      srtp_dealloc (filter->session);
+      filter->session = NULL;
+    }
 
     g_hash_table_remove_all (filter->ssrcs_set);
   }
@@ -1374,8 +1376,14 @@ gst_srtp_enc_change_state (GstElement * element, GstStateChange transition)
           }
         }
       }
-      if ((filter->rtcp_cipher != SRTP_NULL_CIPHER)
-          && (filter->rtcp_auth == SRTP_NULL_AUTH)) {
+
+      /* RFC 3711 says in "3. SRTP Framework" that SRTCP message authentication
+       * is MANDATORY. In case of GCM let the pipeline handle any errors.
+       */
+      if (filter->rtcp_cipher != GST_SRTP_CIPHER_AES_128_GCM
+          && filter->rtcp_cipher != GST_SRTP_CIPHER_AES_256_GCM
+          && filter->rtcp_cipher != GST_SRTP_CIPHER_NULL
+          && filter->rtcp_auth == GST_SRTP_AUTH_NULL) {
         GST_ERROR_OBJECT (filter,
             "RTCP authentication can't be NULL if encryption is not NULL.");
         return GST_STATE_CHANGE_FAILURE;
