@@ -56,13 +56,7 @@ struct _GstWebRTCRTPSenderClass
 };
 
 GST_WEBRTC_API
-GstWebRTCRTPSender *        gst_webrtc_rtp_sender_new                   (GArray * send_encodings);
-GST_WEBRTC_API
-GstStructure *              gst_webrtc_rtp_sender_get_parameters        (GstWebRTCRTPSender * sender, gchar * kind);
-/* FIXME: promise? */
-GST_WEBRTC_API
-gboolean                    gst_webrtc_rtp_sender_set_parameters        (GstWebRTCRTPSender * sender,
-                                                                         GstStructure * parameters);
+GstWebRTCRTPSender *        gst_webrtc_rtp_sender_new                   (void);
 
 GST_WEBRTC_API
 void                        gst_webrtc_rtp_sender_set_transport         (GstWebRTCRTPSender * sender,
@@ -72,6 +66,10 @@ void                        gst_webrtc_rtp_sender_set_rtcp_transport    (GstWebR
                                                                          GstWebRTCDTLSTransport * transport);
 
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstWebRTCRTPSender, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 #endif /* __GST_WEBRTC_RTP_SENDER_H__ */
