@@ -19,17 +19,17 @@
 
 /**
  * SECTION:element-gdpdepay
+ * @title: gdpdepay
  * @see_also: gdppay
  *
  * This element depayloads GStreamer Data Protocol buffers back to deserialized
  * buffers and events.
  *
- * <refsect2>
  * |[
  * gst-launch-1.0 -v -m filesrc location=test.gdp ! gdpdepay ! xvimagesink
  * ]| This pipeline plays back a serialized video stream as created in the
  * example for gdppay.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -560,6 +560,11 @@ gst_gdp_depay_decide_allocation (GstGDPDepay * gdpdepay)
     return;
   }
 
+  if (!gst_caps_is_fixed (caps)) {
+    GST_LOG_OBJECT (gdpdepay, "Caps on src pad are not fixed. Not querying.");
+    return;
+  }
+
   query = gst_query_new_allocation (caps, TRUE);
   if (!gst_pad_peer_query (gdpdepay->srcpad, query)) {
     GST_WARNING_OBJECT (gdpdepay, "Peer allocation query failed.");
