@@ -33,6 +33,20 @@
 #include "mpegtsbase.h"
 #include "mpegtspacketizer.h"
 
+/* color specifications for JPEG 2000 stream over MPEG TS */
+typedef enum
+{
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_UNKNOWN,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_SRGB,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_REC601,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_REC709,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_CIELUV,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_CIEXYZ,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_REC2020,
+  GST_MPEGTSDEMUX_JPEG2000_COLORSPEC_SMPTE2084
+} GstMpegTsDemuxJpeg2000ColorSpec;
+
+
 G_BEGIN_DECLS
 #define GST_TYPE_TS_DEMUX \
   (gst_ts_demux_get_type())
