@@ -250,9 +250,7 @@ mxf_uuid_hash (const MXFUUID * uuid)
   g_return_val_if_fail (uuid != NULL, 0);
 
   for (i = 0; i < 4; i++)
-    ret ^= (uuid->u[i * 4 + 0] << 24) |
-        (uuid->u[i * 4 + 1] << 16) |
-        (uuid->u[i * 4 + 2] << 8) | (uuid->u[i * 4 + 3] << 0);
+    ret ^= GST_READ_UINT32_BE (uuid->u + i * 4);
 
   return ret;
 }
@@ -326,11 +324,12 @@ mxf_uuid_array_parse (MXFUUID ** array, guint32 * count, const guint8 * data,
 
   g_return_val_if_fail (array != NULL, FALSE);
   g_return_val_if_fail (count != NULL, FALSE);
-  g_return_val_if_fail (data != NULL, FALSE);
 
   if (size < 8)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   element_count = GST_READ_UINT32_BE (data);
   data += 4;
   size -= 4;
@@ -351,7 +350,7 @@ mxf_uuid_array_parse (MXFUUID ** array, guint32 * count, const guint8 * data,
     return FALSE;
   }
 
-  if (16 * element_count < size) {
+  if (element_count > size / 16) {
     *array = NULL;
     *count = 0;
     return FALSE;
@@ -492,7 +491,6 @@ mxf_umid_init (MXFUMID * umid)
 gboolean
 mxf_timestamp_parse (MXFTimestamp * timestamp, const guint8 * data, guint size)
 {
-  g_return_val_if_fail (data != NULL, FALSE);
   g_return_val_if_fail (timestamp != NULL, FALSE);
 
   memset (timestamp, 0, sizeof (MXFTimestamp));
@@ -500,6 +498,8 @@ mxf_timestamp_parse (MXFTimestamp * timestamp, const guint8 * data, guint size)
   if (size < 8)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   timestamp->year = GST_READ_UINT16_BE (data);
   timestamp->month = GST_READ_UINT8 (data + 2);
   timestamp->day = GST_READ_UINT8 (data + 3);
@@ -597,13 +597,14 @@ gboolean
 mxf_fraction_parse (MXFFraction * fraction, const guint8 * data, guint size)
 {
   g_return_val_if_fail (fraction != NULL, FALSE);
-  g_return_val_if_fail (data != NULL, FALSE);
 
   memset (fraction, 0, sizeof (MXFFraction));
 
   if (size < 8)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   fraction->n = GST_READ_UINT32_BE (data);
   fraction->d = GST_READ_UINT32_BE (data + 4);
 
@@ -669,13 +670,14 @@ mxf_product_version_parse (MXFProductVersion * product_version,
     const guint8 * data, guint size)
 {
   g_return_val_if_fail (product_version != NULL, FALSE);
-  g_return_val_if_fail (data != NULL, FALSE);
 
   memset (product_version, 0, sizeof (MXFProductVersion));
 
   if (size < 9)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   product_version->major = GST_READ_UINT16_BE (data);
   product_version->minor = GST_READ_UINT16_BE (data + 2);
   product_version->patch = GST_READ_UINT16_BE (data + 4);
@@ -769,8 +771,10 @@ mxf_partition_pack_parse (const MXFUL * ul, MXFPartitionPack * pack,
   gchar str[48];
 #endif
 
+  if (size < 84)
+    return FALSE;
+
   g_return_val_if_fail (data != NULL, FALSE);
-  g_return_val_if_fail (size >= 84, FALSE);
 
   memset (pack, 0, sizeof (MXFPartitionPack));
 
@@ -991,12 +995,13 @@ mxf_random_index_pack_parse (const MXFUL * ul, const guint8 * data, guint size,
   guint len, i;
   MXFRandomIndexPackEntry entry;
 
-  g_return_val_if_fail (data != NULL, FALSE);
   g_return_val_if_fail (array != NULL, FALSE);
 
   if (size < 4)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   if ((size - 4) % 12 != 0)
     return FALSE;
 
@@ -1072,13 +1077,14 @@ mxf_index_table_segment_parse (const MXFUL * ul,
   const guint8 *tag_data;
 
   g_return_val_if_fail (ul != NULL, FALSE);
-  g_return_val_if_fail (data != NULL, FALSE);
 
   memset (segment, 0, sizeof (MXFIndexTableSegment));
 
   if (size < 70)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   GST_DEBUG ("Parsing index table segment:");
 
   while (mxf_local_tag_parse (data, size, &tag, &tag_size, &tag_data)) {
@@ -1167,7 +1173,7 @@ mxf_index_table_segment_parse (const MXFUL * ul,
         tag_data += 4;
         tag_size -= 4;
 
-        if (tag_size < len * 6)
+        if (tag_size / 6 < len)
           goto error;
 
         segment->delta_entries = g_new (MXFDeltaEntry, len);
@@ -1216,7 +1222,8 @@ mxf_index_table_segment_parse (const MXFUL * ul,
         tag_data += 4;
         tag_size -= 4;
 
-        if (tag_size < len * 11)
+        if (tag_size / (11 + 4 * segment->slice_count +
+                8 * segment->pos_table_count) < len)
           goto error;
 
         segment->index_entries = g_new0 (MXFIndexEntry, len);
@@ -1289,9 +1296,11 @@ mxf_index_table_segment_reset (MXFIndexTableSegment * segment)
 
   g_return_if_fail (segment != NULL);
 
-  for (i = 0; i < segment->n_index_entries; i++) {
-    g_free (segment->index_entries[i].slice_offset);
-    g_free (segment->index_entries[i].pos_table);
+  if (segment->index_entries) {
+    for (i = 0; i < segment->n_index_entries; i++) {
+      g_free (segment->index_entries[i].slice_offset);
+      g_free (segment->index_entries[i].pos_table);
+    }
   }
 
   g_free (segment->index_entries);
@@ -1434,8 +1443,10 @@ mxf_primer_pack_parse (const MXFUL * ul, MXFPrimerPack * pack,
   guint i;
   guint32 n;
 
+  if (size < 8)
+    return FALSE;
+
   g_return_val_if_fail (data != NULL, FALSE);
-  g_return_val_if_fail (size >= 8, FALSE);
 
   memset (pack, 0, sizeof (MXFPrimerPack));
 
@@ -1447,14 +1458,16 @@ mxf_primer_pack_parse (const MXFUL * ul, MXFPrimerPack * pack,
 
   n = GST_READ_UINT32_BE (data);
   data += 4;
+  size -= 4;
 
   GST_DEBUG ("  number of mappings = %u", n);
 
   if (GST_READ_UINT32_BE (data) != 18)
     goto error;
   data += 4;
+  size -= 4;
 
-  if (size < 8 + n * 18)
+  if (size / 18 < n)
     goto error;
 
   for (i = 0; i < n; i++) {
@@ -1610,18 +1623,21 @@ gboolean
 mxf_local_tag_parse (const guint8 * data, guint size, guint16 * tag,
     guint16 * tag_size, const guint8 ** tag_data)
 {
-  g_return_val_if_fail (data != NULL, FALSE);
-
   if (size < 4)
     return FALSE;
 
+  g_return_val_if_fail (data != NULL, FALSE);
+
   *tag = GST_READ_UINT16_BE (data);
   *tag_size = GST_READ_UINT16_BE (data + 2);
 
-  if (size < 4 + *tag_size)
+  data += 4;
+  size -= 4;
+
+  if (size < *tag_size)
     return FALSE;
 
-  *tag_data = data + 4;
+  *tag_data = data;
 
   return TRUE;
 }
@@ -1645,7 +1661,7 @@ mxf_local_tag_add_to_hash_table (const MXFPrimerPack * primer,
   MXFUL *ul;
 
   g_return_val_if_fail (primer != NULL, FALSE);
-  g_return_val_if_fail (tag_data != NULL, FALSE);
+  g_return_val_if_fail (tag_size == 0 || tag_data != NULL, FALSE);
   g_return_val_if_fail (hash_table != NULL, FALSE);
   g_return_val_if_fail (primer->mappings != NULL, FALSE);
 
@@ -1671,7 +1687,7 @@ mxf_local_tag_add_to_hash_table (const MXFPrimerPack * primer,
     local_tag = g_slice_new0 (MXFLocalTag);
     memcpy (&local_tag->ul, ul, sizeof (MXFUL));
     local_tag->size = tag_size;
-    local_tag->data = g_memdup (tag_data, tag_size);
+    local_tag->data = tag_size == 0 ? NULL : g_memdup (tag_data, tag_size);
     local_tag->g_slice = FALSE;
 
     g_hash_table_insert (*hash_table, &local_tag->ul, local_tag);
