@@ -210,6 +210,8 @@ create_amc_format (GstAmcVideoEnc * encoder, GstVideoCodecState * input_state,
     }
   } else if (strcmp (name, "video/x-vp8") == 0) {
     mime = "video/x-vnd.on2.vp8";
+  } else if (strcmp (name, "video/x-vp9") == 0) {
+    mime = "video/x-vnd.on2.vp9";
   } else {
     GST_ERROR_OBJECT (encoder, "Failed to convert caps(%s/...) to any mime",
         name);
@@ -395,6 +397,8 @@ caps_from_amc_format (GstAmcFormat * amc_format)
     }
   } else if (strcmp (mime, "video/x-vnd.on2.vp8") == 0) {
     caps = gst_caps_new_empty_simple ("video/x-vp8");
+  } else if (strcmp (mime, "video/x-vnd.on2.vp9") == 0) {
+    caps = gst_caps_new_empty_simple ("video/x-vp9");
   }
 
   gst_caps_set_simple (caps, "width", G_TYPE_INT, width,
@@ -451,7 +455,7 @@ gst_amc_video_enc_base_init (gpointer g_class)
   longname = g_strdup_printf ("Android MediaCodec %s", codec_info->name);
   gst_element_class_set_metadata (element_class,
       codec_info->name,
-      "Codec/Encoder/Video",
+      "Codec/Encoder/Video/Hardware",
       longname, "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
   g_free (longname);
 }
@@ -726,7 +730,8 @@ _find_nearest_frame (GstAmcVideoEnc * self, GstClockTime reference_timestamp)
       best_id = id;
 
       /* For frames without timestamp we simply take the first frame */
-      if ((reference_timestamp == 0 && timestamp == 0) || diff == 0)
+      if ((reference_timestamp == 0 && !GST_CLOCK_TIME_IS_VALID (timestamp))
+          || diff == 0)
         break;
     }
   }
@@ -1017,18 +1022,23 @@ process_buffer:
       " flags 0x%08x", idx, buffer_info.size, buffer_info.presentation_time_us,
       buffer_info.flags);
 
+  buf = gst_amc_codec_get_output_buffer (self->codec, idx, &err);
+  if (err) {
+    if (self->flushing) {
+      g_clear_error (&err);
+      goto flushing;
+    }
+    goto failed_to_get_output_buffer;
+  } else if (!buf) {
+    goto got_null_output_buffer;
+  }
+
   frame =
       _find_nearest_frame (self,
       gst_util_uint64_scale (buffer_info.presentation_time_us, GST_USECOND, 1));
 
   is_eos = ! !(buffer_info.flags & BUFFER_FLAG_END_OF_STREAM);
 
-  buf = gst_amc_codec_get_output_buffer (self->codec, idx, &err);
-  if (err)
-    goto failed_to_get_output_buffer;
-  else if (!buf)
-    goto got_null_output_buffer;
-
   flow_ret =
       gst_amc_video_enc_handle_output_frame (self, buf, &buffer_info, frame);
 
