@@ -3,7 +3,7 @@
  * Copyright (C) 2007 Alessandro Decina
  * 
  * Authors:
- *   Alessandro Decina <alessandro@nnva.org>
+ *   Alessandro Decina <alessandro.d@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -63,7 +63,7 @@ typedef struct
   const gchar *description;
 } CamTagMessage;
 
-static CamTagMessage debugmessage[] = {
+static const CamTagMessage debugmessage[] = {
   {TAG_SB, "SB"},
   {TAG_RCV, "RCV"},
   {TAG_CREATE_T_C, "CREATE_T_C"},
