@@ -540,8 +540,8 @@ gst_msdkenc_close_encoder (GstMsdkEnc * thiz)
   GST_DEBUG_OBJECT (thiz, "Closing encoder with context %" GST_PTR_FORMAT,
       thiz->context);
 
-  gst_object_replace ((GstObject **) & thiz->msdk_pool, NULL);
-  gst_object_replace ((GstObject **) & thiz->msdk_converted_pool, NULL);
+  gst_clear_object (&thiz->msdk_pool);
+  gst_clear_object (&thiz->msdk_converted_pool);
 
   if (thiz->use_video_memory)
     gst_msdk_frame_free (thiz->context, &thiz->alloc_resp);
@@ -1214,7 +1214,9 @@ gst_msdkenc_get_surface_from_frame (GstMsdkEnc * thiz,
   GstVideoFrame src_frame, out_frame;
   MsdkSurface *msdk_surface;
   GstBuffer *inbuf;
+#ifndef _WIN32
   GstMemory *mem = NULL;
+#endif
 
   inbuf = frame->input_buffer;
   if (gst_msdk_is_msdk_buffer (inbuf)) {
@@ -1392,36 +1394,64 @@ invalid_frame:
   }
 }
 
+static gboolean
+gst_msdkenc_context_prepare (GstMsdkEnc * thiz)
+{
+  /* Try to find an existing context from the pipeline. This may (indirectly)
+   * invoke gst_msdkenc_set_context, which will set thiz->context. */
+  if (!gst_msdk_context_find (GST_ELEMENT_CAST (thiz), &thiz->context))
+    return FALSE;
+
+  if (thiz->context == thiz->old_context) {
+    GST_INFO_OBJECT (thiz, "Found old context %" GST_PTR_FORMAT
+        ", reusing as-is", thiz->context);
+    return TRUE;
+  }
+
+  GST_INFO_OBJECT (thiz, "Found context %" GST_PTR_FORMAT " from neighbour",
+      thiz->context);
+
+  /* Check GST_MSDK_JOB_VPP and GST_MSDK_JOB_ENCODER together to avoid sharing context
+   * between VPP and ENCODER
+   * Example:
+   * gst-launch-1.0 videotestsrc ! video/x-raw,format=I420 ! msdkh264enc ! \
+   * msdkh264dec ! msdkvpp ! video/x-raw,format=YUY2 ! fakesink
+   */
+  if (!gst_msdk_context_get_job_type (thiz->context) & (GST_MSDK_JOB_VPP |
+          GST_MSDK_JOB_ENCODER)) {
+    gst_msdk_context_add_job_type (thiz->context, GST_MSDK_JOB_ENCODER);
+    return TRUE;
+  }
+
+  /* Found an existing context that's already being used as an encoder, clone
+   * the MFX session inside it to create a new one */
+  {
+    GstMsdkContext *parent_context, *msdk_context;
+
+    GST_INFO_OBJECT (thiz, "Creating new context %" GST_PTR_FORMAT " with "
+        "joined session", thiz->context);
+    parent_context = thiz->context;
+    msdk_context = gst_msdk_context_new_with_parent (parent_context);
+
+    if (!msdk_context) {
+      GST_ERROR_OBJECT (thiz, "Failed to create a context with parent context "
+          "as %" GST_PTR_FORMAT, parent_context);
+      return FALSE;
+    }
+
+    thiz->context = msdk_context;
+    gst_object_unref (parent_context);
+  }
+
+  return TRUE;
+}
+
 static gboolean
 gst_msdkenc_start (GstVideoEncoder * encoder)
 {
   GstMsdkEnc *thiz = GST_MSDKENC (encoder);
 
-  if (gst_msdk_context_prepare (GST_ELEMENT_CAST (thiz), &thiz->context)) {
-    GST_INFO_OBJECT (thiz, "Found context %" GST_PTR_FORMAT " from neighbour",
-        thiz->context);
-
-    if (gst_msdk_context_get_job_type (thiz->context) & GST_MSDK_JOB_ENCODER) {
-      GstMsdkContext *parent_context, *msdk_context;
-
-      parent_context = thiz->context;
-      msdk_context = gst_msdk_context_new_with_parent (parent_context);
-
-      if (!msdk_context) {
-        GST_ERROR_OBJECT (thiz, "Context creation failed");
-        return FALSE;
-      }
-
-      thiz->context = msdk_context;
-      gst_object_unref (parent_context);
-
-      GST_INFO_OBJECT (thiz,
-          "Creating new context %" GST_PTR_FORMAT " with joined session",
-          thiz->context);
-    } else {
-      gst_msdk_context_add_job_type (thiz->context, GST_MSDK_JOB_ENCODER);
-    }
-  } else {
+  if (!gst_msdkenc_context_prepare (thiz)) {
     if (!gst_msdk_context_ensure_context (GST_ELEMENT_CAST (thiz),
             thiz->hardware, GST_MSDK_JOB_ENCODER))
       return FALSE;
@@ -1429,6 +1459,11 @@ gst_msdkenc_start (GstVideoEncoder * encoder)
         thiz->context);
   }
 
+  /* Save the current context in a separate field so that we know whether it
+   * has changed between calls to _start() */
+  gst_object_replace ((GstObject **) & thiz->old_context,
+      (GstObject *) thiz->context);
+
   gst_msdk_context_add_shared_async_depth (thiz->context, thiz->async_depth);
 
   /* Set the minimum pts to some huge value (1000 hours). This keeps
@@ -1452,7 +1487,7 @@ gst_msdkenc_stop (GstVideoEncoder * encoder)
     gst_video_codec_state_unref (thiz->input_state);
   thiz->input_state = NULL;
 
-  gst_object_replace ((GstObject **) & thiz->context, NULL);
+  gst_clear_object (&thiz->context);
 
   return TRUE;
 }
@@ -1538,9 +1573,8 @@ gst_msdkenc_propose_allocation (GstVideoEncoder * encoder, GstQuery * query)
       query);
 }
 
-
 static void
-gst_msdkenc_finalize (GObject * object)
+gst_msdkenc_dispose (GObject * object)
 {
   GstMsdkEnc *thiz = GST_MSDKENC (object);
 
@@ -1548,10 +1582,11 @@ gst_msdkenc_finalize (GObject * object)
     gst_video_codec_state_unref (thiz->input_state);
   thiz->input_state = NULL;
 
-  gst_object_replace ((GstObject **) & thiz->msdk_pool, NULL);
-  gst_object_replace ((GstObject **) & thiz->msdk_converted_pool, NULL);
+  gst_clear_object (&thiz->msdk_pool);
+  gst_clear_object (&thiz->msdk_converted_pool);
+  gst_clear_object (&thiz->old_context);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -1565,7 +1600,7 @@ gst_msdkenc_class_init (GstMsdkEncClass * klass)
   element_class = GST_ELEMENT_CLASS (klass);
   gstencoder_class = GST_VIDEO_ENCODER_CLASS (klass);
 
-  gobject_class->finalize = gst_msdkenc_finalize;
+  gobject_class->dispose = gst_msdkenc_dispose;
 
   element_class->set_context = gst_msdkenc_set_context;
 
